float4   cnt    : register(c0);

struct VS_OUTPUT {
  float4 pos : POSITION;
  float4 tex : TEXCOORD0;
};

VS_OUTPUT vs_main(VS_OUTPUT input) {
  return input;
}

float iso(float3 p) {
  float  x =    (p.x);
  float  y = cos(p.y) * 3;
  float  z =    (p.z) * 0.1;
  float3 ip = float3(17, 15, 8);
  p *= ip;
  float r = cos(x) + cos(y) + cos(z);
  r -=    ( sin(p.x) + sin(p.y) + sin(p.z)) * 0.01;
  r -= log( sin(p.x) + sin(p.y) + sin(p.z)) * 0.005;
  return r;
}

float raycast(float3 eye, float3 dir, float th) {
  float dt = 3;
  float t = 0;
  for(int i = 0; i < 200 && dt > 0.001; i++) {
    float3 temp = eye + dir * t;
    if(iso(temp) < th) {
      t -= dt;
      dt *= 0.1;
    }
    t += dt;
  }
  return t;
}


float4 ps_main(float2 vp : vpos) : color {
  float2 uv = -1 + 2 * vp / (float2(800, 600));
  float time  = cnt.x;
  float flash = 1;
  
  //if(uv.y < -0.65 || uv.y > 0.65) return 0;
  float rotate = (time + sin(time * 0.3)) * 0.1;
  float3x3 mtx = {
    cos(rotate),  sin(rotate), 0,
    sin(rotate), -cos(rotate), 0,
    0,           0,            1
  };
  
  //rotate ray direction
  float3 dir    = normalize(
    float3(uv.x * 1.25 - sin(time * 3) * 0.03,
           uv.y        + cos(time * 4) * 0.03,
           1));
  dir           = mul(mtx, dir).yzx;
  dir           = mul(mtx, dir) / 8;
  
  //cast
  float3 eye    = float3(0, time * 2, 0);
  float  t      = raycast(eye, dir,  -0.7);
  
  //calc normal.
  float3 ip     = eye + dir * t;
  float  iit    = iso(ip);
  float3 h      = float3(0.01, 0, 0);
  float3 n      = normalize(float3(
    iit - iso(ip + h.xyy),
    iit - iso(ip + h.yxy),
    iit - iso(ip + h.yyx)));
  
  //light.
  float3 light  = normalize(float3(1,  -0.5,  1));
  float3 normal = dot(light, n);
  
  //iro
  float  len    = distance(eye + dir * t, eye);
  float4 col    = float4((dir * 0.5) + float3(0.2, 0.7, 0.3) * normal, 1) - t * 0.001;
  return col;
  
}
